#import "Particle.h"

#define screenWidth 480
#define screenHeight 320

@implementation Particle
@synthesize particle;


- (id) initParticleAt: (double) posX
                    y: (double) posY
                width: (double) w
               height: (double) h
                alpha: (double) a
             lifetime: (double) l
              gravity: (double) g
                 wind: (double) wx
                   rX: (double) rx
                   rY: (double) ry
                  col: (UIColor*) color
           parentView: (UIView*) parentView {
    if (self == [super init]) {
        particle = [[UIImageView alloc] initWithFrame:CGRectMake(posX-w/2, posY-h/2, w, h)];
        particle.backgroundColor=color;
        //[self addSubview:particle];
        [parentView addSubview:particle];
        x = posX;
        y = posY;
        rX = rx;
        rY = ry;
        alpha = a;
        lifetime = l;
        step = l;
        gravity = g;
        wind = wx;
        width = w;
        height = h;
        lifetimeHasEnded = NO;
    }
    return self;
}

- (void) setParticleImage: (UIImage*) img {
    particle.image = img;
    particle.backgroundColor=[UIColor clearColor];
    
}

- (void) go {
    if ((step>0)||(lifetime==0)) {
        step--;
        float faktor = (float)step/(float)lifetime;
        
        if (lifetime!=0) {
            particle.alpha=alpha * faktor;
        }
        
        rX += wind;
        rY += gravity;
        
        x = x + rX ;
        y = y + rY;
        
        particle.center=CGPointMake(x, y);
        
        if (
            (particle.frame.origin.x+particle.frame.size.width<0) ||
            (particle.frame.origin.x>screenWidth) ||
            (particle.frame.origin.y+particle.frame.size.height<0) ||
            (particle.frame.origin.y>screenHeight)
            ) {
            
            [self die];
        }
    }
    else [self die];
}

- (bool) detectCollisionWithFrame: (CGRect) sprite {
    bool collision = NO;
    if (CGRectIntersectsRect(sprite, particle.frame)){
        collision = YES;
    }
    return collision;
}


- (CGRect) frame {return particle.frame;}
- (CGPoint) center {return particle.center;}
- (bool) lifetimeHasEnded {return lifetimeHasEnded;}
- (void) die {
    [particle removeFromSuperview];
    lifetimeHasEnded=YES;
}

@end