#import "ViewController.h"
#import "Sprite.h"
#import "Particle.h"
#include "leveldesign.h"

@implementation ViewController
@synthesize canvas, joypad, player, enemy, imageSource, animationSource, enemySource, areaTop, areaBottom, shots, alienshots, backgroundParticles;


// ----------------------------
// --- ausgemustertes Tile  ---
// --- neu gestylt nach vorne  ---
// ----------------------------

- (void) setTile: (int) tile currentPosition: (int) curPos {
    
    // Tile OBEN:
    Sprite *tmpTop = [areaTop objectAtIndex:tile];
    int spriteTopTyp = pattern[patternSequence[level][curPos/tilesProPattern]]
    [curPos%tilesProPattern][0][1];
    double spriteTopPosY = pattern[patternSequence[level][curPos/tilesProPattern]]
    [curPos%tilesProPattern][0][0] -
    tmpTop.frame.size.height/2;
    // Design anpassen
    [tmpTop setSpriteTyp:[imageSource objectAtIndex:spriteTopTyp] typ:0];
    
    // ans Ende der Tilesreihe springen
    [tmpTop moveBy: tilesTotal*tileSize y: spriteTopPosY- tmpTop.center.y];
    
    // Tile UNTEN:
    Sprite *tmpBottom = [areaBottom objectAtIndex:tile];
    int spriteBottomTyp = pattern[patternSequence[level][curPos/tilesProPattern]]
    [curPos%tilesProPattern][1][1];
    double spriteBottomPosY = screenHeight - pattern[patternSequence[level]
                                                     [curPos/tilesProPattern]]
    [curPos%tilesProPattern][1][0] +
    tmpBottom.frame.size.height/2;
    // Design anpassen
    [tmpBottom setSpriteTyp:[imageSource objectAtIndex:spriteBottomTyp] typ:0];
    // ans Ende der Tilesreihe springen
    [tmpBottom moveBy:tilesTotal*tileSize y: spriteBottomPosY-tmpBottom.center.y];
}

// -------------------------
// --- Spiel vorbereiten ---
// -------------------------

- (void) restartLevel {
    // Reset Aliens
    for (int p=0;p<[enemy count];p++) {
        [[enemy objectAtIndex:p] removeFromSuperview];
        //[[enemy objectAtIndex:p] deleteSprite];
    }
    [enemy removeAllObjects];
    
    // alle Schuesse entfernen
    for (int p=0;p<[shots count];p++) {
        [[shots objectAtIndex:p] die];
    }
    [shots removeAllObjects];
    for (int p=0;p<[alienshots count];p++) {
        [[alienshots objectAtIndex:p] die];
    }
    [alienshots removeAllObjects];
    
    // restliche Partikel entfernen
    for (int p=0;p<[backgroundParticles count];p++) {
        [[backgroundParticles objectAtIndex:p] die];
    }
    [backgroundParticles removeAllObjects];
    
    areaCurrentTile = -1;
    for (int i=0; i<tilesTotal; i++) {
        areaCurrentTile++;
        // Tiles chronologisch links ausserhalb des Displays setzen
        // damit sie dann ueber setTile korrekt platziert werden!
        [[areaTop objectAtIndex:i] setCenter: -(tilesTotal-i)*tileSize y:0];
        [[areaBottom objectAtIndex:i] setCenter: -(tilesTotal-i)*tileSize y:0];
        [self setTile:i currentPosition: i];
    }
    [player setCenter:screenWidth/2 y:screenHeight/2];
    [player setAnimationTyp:[animationSource objectAtIndex:spaceship]
                  spriteTyp:0
                   duration:2.0
                     repeat:0];
    playerSpeedX=0;
    playerSpeedY=0;
    zaehler=0;
    normalAction = YES;
}


- (void) loadView {
    
    // Display: Vollbild und Dauerbeleuchtung
    [[UIApplication sharedApplication] setStatusBarHidden:YES];
    [[UIApplication sharedApplication] setIdleTimerDisabled:YES];
    
    // Hintergrund vorbereiten
    canvas = [[UIView alloc] initWithFrame:[[UIScreen mainScreen]
                                            applicationFrame]];
    canvas.backgroundColor=[UIColor blackColor];
    canvas.userInteractionEnabled = YES;
    canvas.multipleTouchEnabled = YES;
    self.view = canvas;
    
    // Breite und Hoehe speichern (vorsicht: landscape!)
    screenWidth = self.view.bounds.size.height;
    screenHeight = self.view.bounds.size.width;
    
    tilesTotal = screenWidth/tileSize + 1;
    
    // Preloader fuer Bilder (wg. Performance)
    imageSource = [[NSMutableArray alloc] init];
    for (int p=0; p<=4; p++) {
        [imageSource addObject:[UIImage imageNamed:
                                [NSString stringWithFormat:@"area%d.png",p]]];
    }
    
    // Preloader fuer Animationen
    animationSource = [[NSMutableArray alloc] init];
    [animationSource addObject:[NSArray arrayWithObjects:
                                [UIImage imageNamed:@"itype0.png"],
                                [UIImage imageNamed:@"itype1.png"],
                                [UIImage imageNamed:@"itype2.png"],
                                [UIImage imageNamed:@"itype3.png"],
                                [UIImage imageNamed:@"itype4.png"],
                                [UIImage imageNamed:@"itype5.png"],
                                [UIImage imageNamed:@"itype6.png"],
                                [UIImage imageNamed:@"itype7.png"],
                                nil]];
    [animationSource addObject:[NSArray arrayWithObjects:
                                [UIImage imageNamed:@"itype_explosion0.png"],
                                [UIImage imageNamed:@"itype_explosion1.png"],
                                [UIImage imageNamed:@"itype_explosion2.png"],
                                [UIImage imageNamed:@"itype_explosion3.png"],
                                [UIImage imageNamed:@"itype_explosion4.png"],
                                [UIImage imageNamed:@"itype_explosion5.png"],
                                [UIImage imageNamed:@"itype_explosion6.png"],
                                [UIImage imageNamed:@"itype_explosion7.png"],
                                [UIImage imageNamed:@"itype_explosion8.png"],
                                [UIImage imageNamed:@"itype_explosion9.png"],
                                [UIImage imageNamed:@"itype_explosion10.png"],
                                nil]];
    
    // Area vorbereiten
    areaTop = [[NSMutableArray alloc] init];
    areaBottom = [[NSMutableArray alloc] init];
    for (int p=0; p<tilesTotal; p++) {
        Sprite *tmpTop = [[Sprite alloc]
                          initWithImage:[imageSource objectAtIndex: 0]
                          spriteTyp:0
                          parentView:canvas];
        Sprite *tmpBottom = [[Sprite alloc]
                             initWithImage:[imageSource objectAtIndex: 0]
                             spriteTyp:0
                             parentView:canvas];
        [areaTop addObject:tmpTop];
        [areaBottom addObject:tmpBottom];
    }
    
    // Gegner vorbereiten
    
    // Alienanimationen in enemySource speichern
    enemySource = [[NSMutableArray alloc] init];
    [enemySource addObject:[NSArray arrayWithObjects:
                            [UIImage imageNamed:@"alien0.png"],
                            nil]];
    [enemySource addObject:[NSArray arrayWithObjects:
                            [UIImage imageNamed:@"alien1.png"],
                            [UIImage imageNamed:@"alien1b.png"],
                            nil]];
    [enemySource addObject:[NSArray arrayWithObjects:
                            [UIImage imageNamed:@"alien2.png"],
                            [UIImage imageNamed:@"alien2a.png"],
                            [UIImage imageNamed:@"alien2b.png"],
                            [UIImage imageNamed:@"alien2c.png"],
                            [UIImage imageNamed:@"alien2d.png"],
                            nil]];
    [enemySource addObject:[NSArray arrayWithObjects:
                            [UIImage imageNamed:@"alien3.png"],
                            nil]];
    [enemySource addObject:[NSArray arrayWithObjects:
                            [UIImage imageNamed:@"alien3b.png"],
                            nil]];
    
    // Spritecontainer vorbereiten und noch nicht fuellen
    enemy = [[NSMutableArray alloc] init];
    
    // Berechnet die maximale Breite, die ein Sprite einnehmen kann, um es
    // in ausreichendem Abstand neben dem Display erscheinen zu lassen
    maxSpriteWidth=0;
    for (int i=0;i<[enemySource count];i++) {
        NSArray *getAnimArray = [enemySource objectAtIndex:i];
        UIImage * tmpPic = [getAnimArray objectAtIndex:0];
        if (maxSpriteWidth<tmpPic.size.width) maxSpriteWidth=tmpPic.size.width;
    }
    // davon die Haelfte, um mit center besser arbeiten zu koennen
    maxSpriteWidth/=2;
    
    // Sinus-Array vorbereiten
    for (float i = 0; i < 628; i++) {
        sinus[(int)i]=sinf(i/100);
    }
    
    // Raumschiff vorbereiten
    player = [[Sprite alloc] initWithImage: [UIImage imageNamed:@"itype0.png"]
                                 spriteTyp:0
                                parentView:canvas];
    [player setAnimationTyp:[animationSource objectAtIndex:spaceship]
                  spriteTyp:0
                   duration:2.0
                     repeat:0];
    [player addMask:2 y:4 width:33 height:13];
    [player addMask:37 y:10 width:19 height:3];
    [player setCenter:100 y:screenHeight/2];
    
    // Joypad
    joypad = [[UIImageView alloc] initWithFrame:CGRectMake(20, screenHeight-120,
                                                           100, 100)];
    joypad.image = [UIImage imageNamed:@"joypad.png"];
    [canvas addSubview:joypad];
    
    
    level = 0;
    areaCurrentTile=tilesProPattern-1;
    
    shots = [[NSMutableArray alloc] init];
    [NSTimer scheduledTimerWithTimeInterval:0.5
                                     target: self
                                   selector:@selector(shoot)
                                   userInfo:nil
                                    repeats:YES];
    
    alienshots = [[NSMutableArray alloc] init];
    
    backgroundParticles = [[NSMutableArray alloc] init];
    [NSTimer scheduledTimerWithTimeInterval:0.5
                                     target: self
                                   selector:@selector(stars)
                                   userInfo:nil
                                    repeats:YES];
    
    
    // Level vorbereiten
    [self restartLevel];
    
    // Timer fuer Game Engine starten (30fps)
    CADisplayLink *callGameEngine = [CADisplayLink displayLinkWithTarget:self
                                                                selector:@selector(gameEngine)];
    [callGameEngine setFrameInterval:2];
    [callGameEngine addToRunLoop:[NSRunLoop currentRunLoop]
                         forMode:NSDefaultRunLoopMode];
}

-(void) shoot {
    if (normalAction) {
        Particle *shoot = [[Particle alloc]
                           initParticleAt:player.frame.origin.x + player.frame.size.width
                           y:player.center.y+2
                           width:20
                           height:1
                           alpha:1.0
                           lifetime:0
                           gravity:0.00
                           wind:0.0
                           rX:5.0
                           rY:0.0
                           col:[UIColor whiteColor] parentView:canvas];
        [shots addObject:shoot];
    }
}

-(void) stars {
    if (normalAction) {
        double starSize = arc4random()%3 + 1;
        Particle *star =
        [[Particle alloc] initParticleAt:screenWidth
                                       y:fmod(arc4random(),screenHeight)
                                   width:1
                                  height:1
                                   alpha:starSize/4
                                lifetime:0
                                 gravity:0.00
                                    wind:0.0
                                      rX:-starSize-0.2
                                      rY:0.0
                                     col:[UIColor grayColor]
                              parentView:canvas];
        [backgroundParticles addObject:star];
    }
}

// -------------------
// --- Tile Engine ---
// -------------------

-(void) tileEngine {
    for (int p=0; p<[areaTop count]; p++) {
        // Area um 1 Pixel nach links scrollen
        Sprite *tmpTop = [areaTop objectAtIndex:p];
        Sprite *tmpBottom = [areaBottom objectAtIndex:p];
        [tmpTop moveBy:-1 y:0];
        [tmpBottom moveBy:-1 y:0];
        if (
            [player detectCollisionWithMask: tmpTop] ||
            [player detectCollisionWithMask: tmpBottom]
            ) {
            normalAction=NO;
            [player setAnimationTyp:[animationSource objectAtIndex:explosion]
                          spriteTyp:0
                           duration:1.0
                             repeat:1];
            [self performSelector:@selector(restartLevel) withObject:nil afterDelay:1.0];
        }
        // wenn aus Display verschwunden, vorne anstellen
        if (tmpTop.center.x + tileSize/2 <= 0) {
            areaCurrentTile++;
            // wenn Level zu Ende: keine Aktion mehr!
            if (areaCurrentTile>=tilesProPattern*patternProLevel-1) normalAction = NO;
            [self setTile: p currentPosition: areaCurrentTile];
        }
    }
}

// --------------------
// --- Enemy Engine ---
// --------------------

-(void) enemyEngine {
    
    // Alle Aliens in Daten-Array
    for (int curEnemy=0; curEnemy<aliensProLevel; curEnemy++) {
        
        int startX=aliens[level][curEnemy][0];
        int startY=aliens[level][curEnemy][1];
        int spriteImage=aliens[level][curEnemy][2];
        int moveTyp=aliens[level][curEnemy][3];
        int needsShotsToDie=aliens[level][curEnemy][4];
        
        if (startX==zaehler) {
            NSArray *getAnimArray = [enemySource objectAtIndex:spriteImage];
            UIImage * tmpPic = [getAnimArray objectAtIndex:0];
            
            Sprite *newEnemy = [[Sprite alloc] initWithImage: tmpPic
                                                   spriteTyp:spriteImage
                                                  parentView:canvas];
            [newEnemy setAnimationTyp:getAnimArray
                            spriteTyp:moveTyp
                             duration:1.0
                               repeat:0];
            [newEnemy setCenter:screenWidth+maxSpriteWidth y:startY];
            [newEnemy setId:curEnemy];
            [newEnemy setShieldPower:needsShotsToDie];
            [enemy addObject:newEnemy];
        }
    }
    
    // Alle sichtbaren Alien in Sprite-Array
    for (int curVisibleEnemy=0; curVisibleEnemy<[enemy count]; curVisibleEnemy++){
        Sprite *visibleEnemy = [enemy objectAtIndex:curVisibleEnemy];
        
        int startX=aliens[level][[visibleEnemy sId]][0];
        int startY=aliens[level][[visibleEnemy sId]][1];
        
        int posX = visibleEnemy.center.x;
        int posY = visibleEnemy.center.y;
        
        switch ([visibleEnemy spriteTyp]) {
            case wobble:
                posX = posX-3;
                posY = startY+20*sinus[(posX*4)%628];
                
                // Streubomben
                if (posX%30 <= 2) {
                    Particle *shoot =
                    [[Particle alloc] initParticleAt:visibleEnemy.frame.origin.x
                                                   y:visibleEnemy.center.y
                                               width:5
                                              height:5
                                               alpha:1.0
                                            lifetime:50
                                             gravity:0.1
                                                wind:0.0
                                                  rX:-5.0
                                                  rY:0.0
                                                 col:[UIColor yellowColor]
                                          parentView:canvas];
                    [alienshots addObject:shoot];
                }
                
                break;
            case jumper:
                posX = posX-3;
                posY = startY-100*sinus[((int)(posX*2)+314)%314]; // nur Springen
                
                // einfache Schuesse
                if (zaehler%50 == 0) {
                    Particle *shoot =
                    [[Particle alloc] initParticleAt:visibleEnemy.frame.origin.x
                                                   y:visibleEnemy.center.y
                                               width:40
                                              height:5
                                               alpha:1.0
                                            lifetime:200
                                             gravity:0.0
                                                wind:0.0
                                                  rX:-8.0
                                                  rY:0.0
                                                 col:[UIColor redColor]
                                          parentView:canvas];
                    [alienshots addObject:shoot];
                }
                
                break;
            case jumper2:
                posX = posX-3;
                posY = startY-200*sinus[((int)(posX*2)+314)%314]; // nur Springen
                // Torpedostorm
                if (zaehler%4 == 0) {
                    Particle *shoot =
                    [[Particle alloc] initParticleAt:visibleEnemy.frame.origin.x
                                                   y:visibleEnemy.center.y
                                               width:4
                                              height:4
                                               alpha:1.0
                                            lifetime:0
                                             gravity:0.25
                                                wind:-0.5
                                                  rX:5.0
                                                  rY:-4.0
                                                 col:[UIColor blueColor]
                                          parentView:canvas];
                    [alienshots addObject:shoot];
                }
                break;
            case  guerilla:
                posX = posX - 1; // = Tile-Geschwindigkeit
                if (
                    (posX > screenWidth - 100) &&
                    (posX < screenWidth - 50 ) &&
                    (posY > startY-50)
                    ) posY = posY-1;
                if (
                    (posX < 250) && (posY < startY)) posY = posY+1;
                // diagonal
                if (zaehler%20 == 1) {
                    Particle *shoot =
                    [[Particle alloc] initParticleAt:visibleEnemy.frame.origin.x+5
                                                   y:visibleEnemy.frame.origin.y+5
                                               width:3
                                              height:3
                                               alpha:1.0
                                            lifetime:0
                                             gravity:0.0
                                                wind:0.0
                                                  rX:-4.0
                                                  rY:-2.0
                                                 col:[UIColor orangeColor]
                                          parentView:canvas];
                    [alienshots addObject:shoot];
                }
                break;
            case twist:
                posX = screenWidth + maxSpriteWidth - (zaehler - startX) -
                50*sinus[(zaehler*10)%628];
                posY = startY + 50 * sinus[((zaehler+78)*10)%628];
                break;
            case twistFollow1:
                posX = screenWidth + maxSpriteWidth - (zaehler - startX) -
                50*sinus[(628+(zaehler-3)*10)%628];
                posY = startY + 50 * sinus[(628+(zaehler-3+78)*10)%628];
                break;
            case twistFollow2:
                posX = screenWidth + maxSpriteWidth - (zaehler - startX) -
                50*sinus[(628+(zaehler-5)*10)%628];
                posY = startY + 50 * sinus[(628+(zaehler-5+78)*10)%628];
                // Mine
                if (zaehler%10 == 0) {
                    Particle *shoot =
                    [[Particle alloc] initParticleAt:visibleEnemy.frame.origin.x
                                                   y:visibleEnemy.center.y
                                               width:2
                                              height:2
                                               alpha:1.0
                                            lifetime:50
                                             gravity:0.0
                                                wind:0.0
                                                  rX:0.0
                                                  rY:0.0
                                                 col:[UIColor whiteColor]
                                          parentView:canvas];
                    [alienshots addObject:shoot];
                }
                break;
            case bigtwist:
                posX = screenWidth + maxSpriteWidth - 2 * (zaehler - startX) -
                100*sinus[(zaehler*10)%628];
                posY = startY + 100 * sinus[((zaehler+78)*10)%628];
                break;
            case bigtwistFollow1:
                posX = screenWidth + maxSpriteWidth - 2 * (zaehler - startX) -
                100*sinus[(628+(zaehler-1)*10)%628];
                posY = startY + 100 * sinus[(628+(zaehler-1+78)*10)%628];
                break;
            case bigtwistFollow2:
                posX = screenWidth + maxSpriteWidth - 2 * (zaehler - startX) -
                100*sinus[(628+(zaehler-2)*10)%628];
                posY = startY + 100 * sinus[(628+(zaehler-2+78)*10)%628];
                break;
            case attack:
                posX = posX-2;
                if (player.center.y>posY) posY+=1; else posY-=1;
                // wurfgeschoss
                if (zaehler%20 == 1) {
                    Particle *shoot =
                    [[Particle alloc] initParticleAt:visibleEnemy.frame.origin.x
                                                   y:visibleEnemy.center.y
                                               width:3
                                              height:3
                                               alpha:1.0
                                            lifetime:0
                                             gravity:0.1
                                                wind:0.0
                                                  rX:-4.0
                                                  rY:-3.0
                                                 col:[UIColor yellowColor]
                                          parentView:canvas];
                    [alienshots addObject:shoot];
                }
                break;
                
            default:
                break;
        }
        // Alien positionieren
        [visibleEnemy setCenter:posX y:posY];
        
        // Kollisionsabfrage Alien
        if ([player detectCollisionWithMask: visibleEnemy]) {
            normalAction=NO;
            [player setAnimationTyp:[animationSource objectAtIndex:explosion]
                          spriteTyp:0
                           duration:1.0
                             repeat:1];
            [self performSelector:@selector(restartLevel)
                       withObject:nil
                       afterDelay:1.0];
        }
        
        // Alien entfernen, wenn aus Display raus
        if (visibleEnemy.center.x<-visibleEnemy.frame.size.width/2) {
            [visibleEnemy removeFromSuperview];
            [enemy removeObject:visibleEnemy];
        }
        
    }
    
    // Handling Alien Schuesse
    for (int i=0;i<[alienshots count];i++) {
        Particle* shot=[alienshots objectAtIndex:i];
        [shot go];
        
        // Kollisionsabfrage Alien Schuesse
        if ([shot detectCollisionWithFrame:[player frame]]) {
            normalAction=NO;
            [player setAnimationTyp:[animationSource objectAtIndex:explosion]
                          spriteTyp:0
                           duration:1.0
                             repeat:1];
            [self performSelector:@selector(restartLevel)
                       withObject:nil
                       afterDelay:1.0];
        }
        
        // Schuesse: Partikel loeschen
        if ([shot lifetimeHasEnded]) {
            [alienshots removeObject:shot];
        }
    }
    
    
}


- (void)shotEngine {
    for (int i=[shots count]-1;i>=0;i--) {
        Particle* shot = [shots objectAtIndex:i];
        
        [shot go];
        
        // Kollisionsabfrage mit Aliens
        int visibleAliens=[enemy count];
        for (int p=0;p<visibleAliens;p++) {
            Sprite * hitAlien = [enemy objectAtIndex:p];
            if ([shot detectCollisionWithFrame:[hitAlien frame]]) {
                // Schuss entfernen
                [shot die];
                
                // Schutzschild schwaechen
                [hitAlien setShieldPower:[hitAlien shieldPower]-1];
                
                // wenn Schutzschirm erloschen: getroffenes Alien entfernen
                if ([hitAlien shieldPower]==0) {
                    
                    // Explosionspartikel
                    for (int z=0;z<20;z++) {
                        Particle* explosive =
                        [[Particle alloc] initParticleAt:hitAlien.center.x
                                                       y:hitAlien.center.y
                                                   width:5
                                                  height:5
                                                   alpha:1.0
                                                lifetime:arc4random()%20+30
                                                 gravity:0.1
                                                    wind:-0.1
                                                      rX:fmod(arc4random(),6.1)-4.0
                                                      rY:fmod(arc4random(),6.1)-3
                                                     col:[UIColor grayColor]
                                              parentView:canvas];
                        [explosive setParticleImage:[UIImage imageNamed:@"spark.png"]];
                        [backgroundParticles addObject:explosive];
                    }
                    
                    
                    [hitAlien removeFromSuperview];
                    [enemy removeObject:hitAlien];
                    
                    visibleAliens--;
                }
            }
        }
        
        if ([shot lifetimeHasEnded]) {
            [shots removeObject:shot];
        }
    }
}

// sonstige Partikelaktionen (Sterne, Explosionen etc.)
- (void)bgParticles {
    for (int i=[backgroundParticles count]-1;i>=0;i--) {
        Particle* bg = [backgroundParticles objectAtIndex:i];
        [bg go];
        if ([bg lifetimeHasEnded]) [backgroundParticles removeObject:bg];
    }
}

// -------------------
// --- Game Loop -----
// -------------------

-(void) gameEngine {
    
    if (normalAction) {
        
        // Zaehler fuer Animationen
        zaehler++;
        
        [self tileEngine];
        [self enemyEngine];
        [self shotEngine];
        [self bgParticles];
        
        [player moveBy:playerSpeedX y: playerSpeedY];
        
        // z-Indizes sortieren
        [self.view bringSubviewToFront:player];
        [self.view bringSubviewToFront:joypad];
        
    }
}    

- (void) touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
    [self touchesMoved:touches withEvent:event];
}

- (void) touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
    NSArray *allTouches = [touches allObjects]; 
    CGPoint coord = [[allTouches objectAtIndex:[touches count]-1]
                     locationInView:self.view];
    
    if (
        (coord.x-joypad.center.x)*(coord.x-joypad.center.x) +
        (coord.y-joypad.center.y)*(coord.y-joypad.center.y)
        <= (50 * 50)
        ) {
        playerSpeedX = (coord.x-joypad.center.x)/7;
        playerSpeedY = (coord.y-joypad.center.y)/7;
    }
}

- (void) touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    NSArray *allTouches = [touches allObjects]; 
    
    CGPoint coord = [[allTouches objectAtIndex:[allTouches count]-1]
                     locationInView:self.view];
    if (
        (coord.x-joypad.center.x)*(coord.x-joypad.center.x) +
        (coord.y-joypad.center.y)*(coord.y-joypad.center.y)
        <= (50 * 50)
        ) {
        playerSpeedX = 0;
        playerSpeedY = 0;
    }
}



- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return (interfaceOrientation == UIInterfaceOrientationLandscapeLeft);
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

- (void)viewDidUnload {
}

@end